/**
 * @file verification.hpp
 *
 * @brief General Verification type definitions
 */


#include <utility>
#include <vector>

#include <boost/dynamic_bitset.hpp>

#include <core/circuit.hpp>
#include <core/functor.hpp>


namespace revkit
{
  /**
   * @brief Datastructure for counterexamples. 
   *
   * @author RevKit
   * @since  1.0
   */
  typedef std::vector < std::pair < boost::dynamic_bitset<>, boost::dynamic_bitset<> > >  counterexample;

    /**
     * @brief Functor for equivalence checker functions 
     *
     * @author RevKit
     * @since  1.0
     */
  typedef functor<bool (circuit const&, circuit const& impl)> equivalence_func;
}
