/* RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
 * Copyright (C) 2009-2010  The RevKit Developers <revkit@informatik.uni-bremen.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file copy_metadata.hpp
 *
 * Copy meta-data from specification or circuit to circuit
 */

#ifndef COPY_METADATA_HPP
#define COPY_METADATA_HPP

#include "../circuit.hpp"
#include "../truth_table.hpp"

namespace revkit
{

  /**
   * @brief Copies meta-data from a specification to a circuit
   *
   * This method reads the inputs and outputs from a specification
   * and assigns to it to a circuit. Truth-table based synthesis
   * algorithms should use this method.
   *
   * @param spec Truth Table
   * @param circ Circuit
   *
   * @author RevKit
   * @since  1.0
   */
  template<typename T>
  void copy_metadata( const truth_table<T>& spec, circuit& circ )
  {
    circ.set_inputs( spec.inputs() );
    circ.set_outputs( spec.outputs() );
    circ.set_constants( spec.constants() );
    circ.set_garbage( spec.garbage() );
  }

  /**
   * @brief Copies meta-data from a circuit to another circuit
   *
   * This method copies everything but the gates from circuit \p base
   * to circuit \p circ.
   *
   * @param base Source circuit
   * @param circ Destination circuit
   *
   * @author RevKit
   * @since  1.0
   */
  void copy_metadata( const circuit& base, circuit& circ );

}

#endif /* COPY_METADATA_HPP */
