#!/usr/bin/python

# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2010  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from revkit import *

opts = program_options()
opts.add_read_realization_option()
opts.add_write_realization_option()
opts.add_option( "max_window_lines", 6, "Initial maximum size for windows to be considered" ) \
    .add_option( "max_grow_up_window_lines", 9, "If no valid truth table can be found for max_window_lines size windows, they can grow up to this size" ) \
    .add_option( "window_variables_threshold", 17, "Number of primary inputs for pre-window-simulation which still allow the use of simple simulation. Afterwards, SMT - based simulation is used." )

opts.parse( sys.argv )

if not opts.good():
    print opts
    exit( 1 )

circ = circuit()
read_realization( circ, opts.read_realization_filename() ) or sys.exit ("Cannot read " + opts.read_realization_filename())

newcirc = circuit()

r = line_reduction( newcirc, circ, \
                        max_window_lines = opts["max_window_lines"], \
                        max_grow_up_window_lines = opts["max_grow_up_window_lines"], \
                        window_variables_threshold = opts["window_variables_threshold"] )

if opts.is_write_realization_filename_set():
    write_realization( newcirc, opts.write_realization_filename() )

print "Original Circuit:"
print_statistics( circ )

print
print "Optimized Circuit:"
ps_settings = print_statistics_settings()
ps_settings.main_template += "\nConsidered Windows:               " + str( r['num_considered_windows'] ) \
                           + "\nNo suiteable constant line found: " + str( r['statistics.skipped_no_constant_line'] ) \
                           + "\nSkipped due to ambiguous lines:   " + str( r['statistics.skipped_ambigious_line'] ) \
                           + "\nSkipped due to max window lines:  " + str( r['statistics.skipped_max_window_lines'] ) \
                           + "\nSkipped due to synthesis fail:    " + str( r['statistics.skipped_synthesis_failed'] ) + "\n"
print_statistics( newcirc, statistics.runtime, ps_settings )
